//  X-Sim Sample code for receiving OBD2 values from the extractor without using the converter
//  OBD2 is industry standard, X-Sim will additionally expand the OBD2 PID with gear and other values
//
//  Usage:
//  Arduino has to be connected to a free USB port on the computer where the extractor is running
//  Open the extractor and open the settings menu, there you have to select the OBD2 menu
//  Add now your arduino comport to the OBD2 list.
//  After closing the dialog you will see the arduino LED will be enabled which represents receiving data.
//  After closing the extractor application the LED will switch off.
//  Use this code to insert your own display code at the fitting positions
//  Copyright 2013 Martin Wiedenbauer, X-Sim.de
//
//  References:
//  http://en.wikipedia.org/wiki/OBD-II_PIDs
//  http://elmelectronics.com/DSheets/ELM327DS.pdf
//  http://www.x-sim.de


#include <TM1638.h> //can be downloaded from http://code.google.com/p/tm1638-library/

// define a module on data pin 5, clock pin 4 and strobe pin 3
TM1638 module1(5, 4, 3);

//définition des broches du décodeur 7 segments (vous pouvez changer
//les numéros si bon vous semble)
const int bit_A= 6;
const int bit_B= 7;
const int bit_C= 8;
const int bit_D= 9;
//Definition des Pin pour Affichage Neutral et reverse
const int bit_E= 10;
const int bit_F= 11;
//const int ledPin =  13;   // the number of the LED pin
//byte buttons;
unsigned int rpmmaxi=5000;
//long previousMillis = 0;        // will store last time LED was updated

// the follow variables is a long because the time, measured in miliseconds,
// will quickly become a bigger number than can be stored in an int.
//long interval = 40;           // interval at which to blink (milliseconds)

bool extractordetected=false;	//Will get updated from last positive or negative receive
int receivebuffer[20]={0};      //Receive buffer


void setup()
{
	pinMode(13, OUTPUT); //Arduino UNO LED off
	digitalWrite(13, LOW);
	Serial.begin(9600);
        //Do here stuff to init display and zero to default
//pinMode(ledPin, OUTPUT);
//Les broches sont toutes des sorties
pinMode(bit_A, OUTPUT);
pinMode(bit_B, OUTPUT);
pinMode(bit_C, OUTPUT);
pinMode(bit_D, OUTPUT);
pinMode(bit_E, OUTPUT);
pinMode(bit_F, OUTPUT);

//Les broches sont toutes mises à l'état bas pour le decodeur BCD et haut pour piloter neutral et reverse
digitalWrite(bit_A, LOW);
digitalWrite(bit_B, LOW);
digitalWrite(bit_C, LOW);
digitalWrite(bit_D, LOW);
digitalWrite(bit_E, HIGH);
digitalWrite(bit_F, HIGH);

// initialize the screen:
module1.clearDisplay();              //clears the display from garbage if any
String name = "obd2 com";           //sets a custom logo start up banner
module1.setDisplayToString(name);    //prints the banner
name = "xsim3";           //sets a custom logo start up banner

module1.setLEDs(0b10000000 | 0b00000001<< 8 );
delay(50);
module1.setLEDs(0b11000000 | 0b00000011<< 8 );
delay(50);
module1.setLEDs(0b11100000 | 0b00000111<< 8 );
delay(50);
module1.setLEDs(0b11110000 | 0b00001111<< 8 );
delay(50);
module1.setLEDs(0b11111000 | 0b00011111<< 8 );
delay(50);
module1.setLEDs(0b11111100 | 0b00111111<< 8 );
delay(50);
module1.setLEDs(0b11111110 | 0b01111111<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(200);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(100);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(100);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(100);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(3000);                        //small delay 3 sec 
module1.clearDisplay();              //clears the 1st display
}
int HexToInt(int c)
{
	if (c >= '0' && c <= '9') 
	{
		return c - '0';
	} 
	else if (c >= 'a' && c <= 'f') 
	{
		return c - 'a' + 10;
	} 
	else if (c >= 'A' && c <= 'F') 
	{
		return c - 'A' + 10;
	} 
	else 
	{
		return -1; // getting here is bad: it means the character was invalid
	}
}

int ParseReceiveBuffer(int commandhighbyte, int commandlowbyte, int receivelength)
{
	//First character is removed of the receivebuffer string and must not be verified again
	if( receivebuffer[0]=='1' && receivebuffer[2]==commandhighbyte && receivebuffer[3]==commandlowbyte )
	{
		//Parse 2 byte values on position 5 and 6 in the buffer string
		if(receivelength==7)
		{
			int highresult=HexToInt(receivebuffer[5]);
			int lowresult =HexToInt(receivebuffer[6]);
			if(highresult==-1 || lowresult==-1){return -1;}
			return ((16*highresult) + lowresult);
		}
		//Parse 4 byte values on position 5,6,8 and 9 in the buffer string
		if(receivelength==10)
		{
			int tophighresult=HexToInt(receivebuffer[5]);
			int toplowresult =HexToInt(receivebuffer[6]);
			int highresult=HexToInt(receivebuffer[8]);
			int lowresult =HexToInt(receivebuffer[9]);
			if(tophighresult==-1 || toplowresult==-1 || highresult==-1 || lowresult==-1){return -1;}
			return ((4096*tophighresult) + (256*toplowresult) + (16*highresult) + lowresult);
		}
	}
	return -1; //Something is wrong with the returned OBD2 echo command byte
}

//This function will wait for the first character in receivetrigger and will parse the result
int ReceiveValueWithTimeout(int receivetrigger, int commandhighbyte, int commandlowbyte, int receivelength)
{
	int arduinoserialbuffer=0;
	int buffercount=-1;
	for(int z=0; z < 1500; z++) //500ms timeout should be enough
	{
		while(Serial.available())
		{
			if(buffercount==-1)
			{
				arduinoserialbuffer = Serial.read();
				if(arduinoserialbuffer != receivetrigger) //Wait until the trigger is reached, ignore echo, first character is not in the buffer
				{
					buffercount=-1;
				}
				else
				{
					buffercount=0;
				}
			}
			else
			{
				arduinoserialbuffer = Serial.read();
				receivebuffer[buffercount]=arduinoserialbuffer;
				buffercount++;
				if(buffercount > receivelength) //buffer has now waitlen character length
				{
					return ParseReceiveBuffer(commandhighbyte, commandlowbyte, receivelength);
					buffercount=-1;
				}
			}
		}
		delay(1);
	}
	return -1;
}

void SendEchoDisabled() //not used here and a part of the OBD2 ELM327 specifications, as INFO
{
	//ATE0 Echo disabled
	Serial.write('A');
	Serial.write('T');
	Serial.write('E');
	Serial.write('0');
	Serial.write('\r');
}

//010c Request RPM, remember:  OBD2 RPM is ((A*256)+B)/4
int GetOBD2RpmValue()
{
	//010c
	Serial.write('0');
	Serial.write('1');
	Serial.write('0');
	Serial.write('c');
	Serial.write('\r');
	return ReceiveValueWithTimeout('4','0','C',10);
}

//01e9 Request speed in kmh
int GetOBD2SpeedValue()
{
	//01e9
	Serial.write('0');
	Serial.write('1');
	Serial.write('e');
	Serial.write('9');
	Serial.write('\r');
	return ReceiveValueWithTimeout('4','E','9',10);
}

//01e0 Request gear number
int GetOBD2GearValue()
{
	//01e0
	Serial.write('0');
	Serial.write('1');
	Serial.write('e');
	Serial.write('0');
	Serial.write('\r');
	return ReceiveValueWithTimeout('4','E','0',7);
}
//01e1 Request position value
int GetOBD2positionValue()
{
	//01e1
	Serial.write('0');
	Serial.write('1');
	Serial.write('e');
	Serial.write('1');
	Serial.write('\r');
	return ReceiveValueWithTimeout('4','E','1',7);
}
//function to display the gear on the big 7 segmemts display
void afficher(char chiffre)
{
  // pin are all set to lower levelet high level for neutral reverse
  digitalWrite(bit_A, LOW);
  digitalWrite(bit_B, LOW);
  digitalWrite(bit_C, LOW);
  digitalWrite(bit_D, LOW);
  digitalWrite(bit_E, HIGH);
  digitalWrite(bit_F, HIGH);
  //display the numerical needed
  if(chiffre >= 8)
  {
    digitalWrite(bit_D, HIGH);
    chiffre= chiffre - 8;
    }
  if(chiffre>= 4)
  {
    digitalWrite(bit_C, HIGH);
  chiffre= chiffre - 4;
  }
  if(chiffre >= 2)
  {
    digitalWrite(bit_B, HIGH);
    chiffre= chiffre - 2;
  }
  if(chiffre >= 1)
  {
    digitalWrite(bit_A, HIGH);
    chiffre= chiffre - 1;
  }
}
void loop()
{
        //unsigned long currentMillis = millis();
        int i ;
        unsigned int rpmleds; //holds the 8 leds values
        char message [8];     // variable to display the message on the display
	int vitesse=0;
	unsigned int rpm; 
	int gear=GetOBD2GearValue();
	while(gear >= 0)
	{
	//unsigned long currentMillis = millis();	
            if(extractordetected==false)
		{
			SendEchoDisabled();	//Will cause a OK\r as answer from the extractor, ignored in this code
			extractordetected=true;
		}
		//Read all values
		gear = GetOBD2GearValue(); //Notice: offset +1 because of reverse gear
		vitesse = GetOBD2SpeedValue();
		rpm  = GetOBD2RpmValue();
                rpm = rpm/4;
                //unsigned long currentMillis = millis();
                sprintf(message,"%3d%5d",vitesse,rpm);//display the message first 3 digits speed last 5 digits rpm
                
		if(gear!=-1 && vitesse!=-1 && rpm!=-1)
		{
			digitalWrite(13, HIGH); //We have connection and all values are ok, turn on the LED
			//Do here your LCD or display stuff
                     //rpm = constrain (rpm , 0, rpmmaxi);// limit rpm to rpmmaxi
                     rpmleds = map(rpm,0,rpmmaxi-300,0,5); // distributes the rpm level to the 8 leds + 1 for shift change
                                                           //the shift change occurs at rpmmaxi-300rpm
                        if (rpmleds==0){ 
                          module1.setLEDs(0b00000000 | 0b00000000 << 8);
                        }
                        if (rpmleds==1){
                          module1.setLEDs(0b00000000 | 0b00011000 << 8 );
                        }
                        if (rpmleds==2){
                          module1.setLEDs(0b00000000 | 0b00111100 << 8 );
                        }
                        if (rpmleds==3){
                          module1.setLEDs(0b00000000 | 0b01111110 << 8 );
                        }
                        if (rpmleds==4){
                          module1.setLEDs(0b10000001 | 0b11111111 << 8);
                        }
                        if (rpmleds==5){
                          //module1.setLEDs(0b10000001 | 0b11111111 << 8 );
                        //delay(25);
                         //if (rpm=rpmmaxi-300);{
                          //module1.setLEDs(0x0FF );
                             //if(currentMillis - previousMillis > interval){
                         // save the last time you blinked the LED 
                         //previousMillis = currentMillis;
                         for(i=0;i<5;i++){//for the leds gear shift from left to right i=0 to i=7
                         module1.setLED(TM1638_COLOR_RED,i);//first 4 leds from 1 to 4
                         module1.setLED(TM1638_COLOR_RED,7-i);// last leds from 7 to 5
                         //module1.setupDisplay(true, i);//display intensity
                         delay(25);
                         module1.setLEDs(0x0);//OFF display
                         delay (25);
                            // }
                           }
                         }
                        if (gear==0){
                        // displays the character for reverse
                	digitalWrite(bit_A, HIGH);
                	digitalWrite(bit_B, HIGH);
                	digitalWrite(bit_C, HIGH);
                	digitalWrite(bit_D, HIGH);
                	digitalWrite(bit_E, LOW);
                	digitalWrite(bit_F, LOW);
                        //display speed
                        module1.clearDisplay();
                        module1.setDisplayToString(message , 0,0);
                        //display rpm and blinked led
                        //displays numerical the speed
                        }
                        if (gear==1){//displays the caracter for neutral
                        digitalWrite(bit_A, HIGH);
	                digitalWrite(bit_B, HIGH);
                  	digitalWrite(bit_C, HIGH);
                  	digitalWrite(bit_D, HIGH);
                  	digitalWrite(bit_E, LOW);
                  	digitalWrite(bit_F, HIGH);
                        //display speed
                        module1.clearDisplay();
                        module1.setDisplayToString(message , 0,0);
                        if (module1.getButtons() == 0b10000000){ //store the rpmmaxi with pushing the most right button
                          rpmmaxi=rpm;
                          module1.setDisplayToDecNumber(rpmmaxi, 0, false);//display the rpmmaxi value registered
                          delay (500);
                          }
                          /*if (module1.getButtons() == 0b01000000){// decrease the rpmmaxi with 200rpm pitch
                          rpmmaxi=rpmmaxi-200;
                           module1.setDisplayToDecNumber(rpmmaxi, 0, false);//display the rpmmaxi value registered
                          delay (500);
                          }*/
                        //display rpm and blinked led
                        //displays numerical the speed
                        
                        }
                        if (gear >= 2 and gear <10 ){
                        gear=gear-1;
                        afficher(gear);//call function to display gear
                        module1.clearDisplay();
                        module1.setDisplayToString(message , 0,0);//displays numerical the speed and rpm
                        }   
		}

	}

	extractordetected=false;
	digitalWrite(13, LOW); //No connection, turn off the LED

	//Do here stuff to set display to zero or default
module1.clearDisplay();              //clears the display from garbage if any
String name = "Sim OFF";           //sets a custom logo start up banner
module1.setDisplayToString(name);    //prints the banner
name = "sim";                        //sets a custom logo start up banner
delay (500);
module1.clearDisplay();
module1.setDisplayToDecNumber(rpmmaxi, 0, false);//display the rpmmaxi value registered
delay (500);
    /*if (module1.getButtons() == 0b10000000){ //increase the rpmmaxi with 200rpm pitch
      rpmmaxi=rpmmaxi + 200;
  }
    if (module1.getButtons() == 0b01000000){// decrease the rpmmaxi with 200rpm pitch
      rpmmaxi=rpmmaxi-200;
    }*/
module1.setLEDs(0b10000000 | 0b00000001<< 8 );
delay(50);
module1.setLEDs(0b11000000 | 0b00000011<< 8 );
delay(50);
module1.setLEDs(0b11100000 | 0b00000111<< 8 );
delay(50);
module1.setLEDs(0b11110000 | 0b00001111<< 8 );
delay(50);
module1.setLEDs(0b11111000 | 0b00011111<< 8 );
delay(50);
module1.setLEDs(0b11111100 | 0b00111111<< 8 );
delay(50);
module1.setLEDs(0b11111110 | 0b01111111<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(200);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(100);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(100);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(50);
module1.setLEDs(0b11111111 | 0b11111111<< 8 );
delay(100);
module1.setLEDs(0b00000000 | 0b00000000<< 8 );
delay(1000);                        //small delay 1 sec 
module1.clearDisplay();              //clears the 1st display
}

